//----------------------------------------------------------------------------
// 
//	file: affw_window.cc
// 
//	function: afferent weight display window
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Fri Oct 23 06:04:50 CDT 1998
// 
//	$Revision: 1.1 $ $Date: 2002/06/20 22:36:52 $
//	$Log: affw_window.cc,v $
//	Revision 1.1  2002/06/20 22:36:52  yschoe
//	Initial revision
//
//	Revision 1.2  2001/11/30 20:24:15  yschoe
//	pglissom v1.0 release
//
//	Revision 1.1  1998/10/26 03:45:33  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------
#include "affw_window.h"

//----------------------------------------------------------------------------
//	affw_window() 
//	  - constructor
//----------------------------------------------------------------------------
affw_window::affw_window(char *name, netlayer *map, int width, int height, 
	int cur_inp) 
 : tool_window(name, width, height, 0, 0, 0) {

  init(map,width,height,
	map->feedings[cur_inp]->cols,map->feedings[cur_inp]->rows,cur_inp);

}

//----------------------------------------------------------------------------
// with configurable aff_cols, aff_rows
affw_window::affw_window(char *name, netlayer *map, int width, int height, 
	int aff_cols, int aff_rows, int cur_inp) 
 : tool_window(name, width, height, 0, 0, 0) {

  init(map,width,height,aff_cols,aff_rows,cur_inp);

}

//----------------------------------------------------------------------------
//	init() 
//	  - initialize stuff for constructor
//----------------------------------------------------------------------------
void affw_window::init(netlayer *map, int width, int height,
        int aff_cols, int aff_rows, int cur_inp) {

  // 1. get the dimension of the map and the selected (current) input layer
  cols = map->cols;
  rows = map->rows;

  affw_window::aff_cols = aff_cols;
  affw_window::aff_rows = aff_rows;

  affw_window::map = map;
  affw_window::cur_inp = cur_inp;

  // 2. allocate the matrix : must take into account the 
  //	separating lines (so, for NxN map, it would need (M+1)*N * (M+1)*N).
  sizex = cols*(aff_cols+1);
  sizey = rows*(aff_rows+1);

  multi_matrix = new REAL* [sizex];
  for (int i=0; i<sizex; ++i) 
	multi_matrix[i] = new REAL [sizey];

  buffer_matrix = new REAL* [aff_cols];
  for (int i=0; i<aff_cols; ++i) 
	buffer_matrix[i] = new REAL [aff_rows];
	

  // 3. insert a matrix window linked to this matrix in this window.
  matw = new matrix_window(*this,width,height,0,0,0,60,COLOR,
			sizex,sizey,multi_matrix);

  // 4. redraw
  redraw();

}

//----------------------------------------------------------------------------
//	~affw_window() 
//	  - destructor
//----------------------------------------------------------------------------
affw_window::~affw_window() {

  // 1. free the matrix : must take into account the 
  //	separating lines (so, for NxN map, it would need (M+1)*N * (M+1)*N).
  for (int i=0; i<sizex; ++i) 
	delete [] multi_matrix[i];
  delete [] multi_matrix;

  for (int i=0; i<aff_cols; ++i) 
	delete [] buffer_matrix[i];
  delete [] buffer_matrix;
  
  delete matw;

}
  
//----------------------------------------------------------------------------
//	redraw() 
//	  - fill in the matrix and redraw
//----------------------------------------------------------------------------
void affw_window::redraw() {

  // 1. draw affw boundaries
  for (int i=0; i<sizex; ++i) {
     for (int j=0; j<sizey; ++j) {

	if ((i+1)%(aff_cols+1)==0 || (j+1)%(aff_rows+1)==0)  
		multi_matrix[i][j] = 1.0;   

     }
  }

  // 2. now fill in with the afferent weights
  for (int i=0; i<cols; ++i) {
     for (int j=0; j<rows; ++j) {
	
	int offset_x = (aff_cols+1)*i;
	int offset_y = (aff_rows+1)*j;

	map->get_affw(buffer_matrix,i,j,cur_inp);
	for (int k=0; k<aff_cols; ++k)
	   for (int l=0; l<aff_rows; ++l)
		multi_matrix[k+offset_x][l+offset_y] = buffer_matrix[k][l];
     }
  }

  matw->redraw();
  XFlush(display);

}


//----------------------------------------------------------------------------
//	set_scale() 
//	  - set display scale for matw
//----------------------------------------------------------------------------
void affw_window::set_scale(REAL s) {

  matw->set_scale(s);

}
