//----------------------------------------------------------------------------
// 
//	file: blackboard.cc
// 
//	function: blackboard interface to variables : features save and restore 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Wed Jul 15 03:18:34 CDT 1998
// 
//	$Revision: 1.13 $ $Date: 2001/11/30 20:24:15 $
//	    $Log: blackboard.cc,v $
//	    Revision 1.13  2001/11/30 20:24:15  yschoe
//	    pglissom v1.0 release
//
//	    Revision 1.12  2000/02/13 10:29:43  yschoe
//	    use hash for searching parameters.
//
//	    Revision 1.11  2000/01/30 14:56:21  yschoe
//	    *** empty log message ***
//
//	    Revision 1.10  1999/02/09 07:37:11  yschoe
//	    added list_scalar()
//
//	    Revision 1.9  1998/10/17 20:09:17  yschoe
//	    fixed nasty bug in set_parameter() -- both ones.
//	      - delete added,
//	      - return 0, return 1 added
//
//	    Revision 1.8  1998/10/16 10:00:57  yschoe
//	    rewrote set_parameter() for a more flexible use from other
//	    classes. Created a new set_parameter(parser* p) and the old
//	    set_parameter(char *) calls this.
//	    Added force_find() which does find() but also displays an error message
//	    when not found.
//
//	    Revision 1.7  1998/10/05 19:48:46  yschoe
//	    fixed function pointer incompatibility for egcs.
//
//	    Revision 1.6  1998/09/04 03:36:37  yschoe
//	    fixed all warnings generated by -Wall
//
//	    Revision 1.5  1998/08/14 06:51:47  yschoe
//	    added update() for REAL **
//
//	    Revision 1.4  1998/08/11 16:53:58  yschoe
//	    minor fixes. Increased buffer size
//
//	    Revision 1.3  1998/07/24 11:23:51  yschoe
//	    list() added. Removed couts
//
//	    Revision 1.2  1998/07/24 09:04:30  yschoe
//	    added list()
//
//	Revision 1.1  1998/07/23 01:07:05  yschoe
//	    jul22 stable version (except netlayer)
//
//
//----------------------------------------------------------------------------

#include "blackboard.h"

//----------------------------------------------------------------------------
//	blackboard() 
//	  - Constructor
//----------------------------------------------------------------------------
blackboard::blackboard(char *comment, int max) {

  count = 0; 			// number of stored variables
  blackboard::max = max;

  strncpy(version,BB_VERSION_STR,strlen(BB_VERSION_STR)+1); // version
  strncpy(blackboard::comment,comment,strlen(comment)+1); // comment

  table = new BB_Entry [max]; // set up blackboard table
  hashtab  = new ENTRY [max]; // hash table entries
  for (int i=0; i<max; ++i) {
	hashtab[i].key = new char [BB_NAME_LEN];
	hashtab[i].data = new char [BB_NAME_LEN];
  }

  // All subsequent initializations of individual blacboard entries
  // must be done by the add() member function each time a variable
  // is added.

}

//----------------------------------------------------------------------------
//	~blackboard() 
//	  - Destructor
//----------------------------------------------------------------------------
blackboard::~blackboard() {

  // Just get rid of the table[]
  delete [] table;
  for (int i=0; i<max; ++i) {
	delete [] (char *)hashtab[i].key;
	delete [] (char *)hashtab[i].data;
  }
  delete [] hashtab;

}

//----------------------------------------------------------------------------
//	init() 
//	  - initialize a single BB_Entry (pointed by bb)
//	  - only need to set the BB_Data pointer after this routine is called 
//	 	size: data type size (sizeof(element))
//----------------------------------------------------------------------------
void blackboard::init(char *name, int size, int cols, int rows, 
	BB_Type type, BB_Entry *bb){

  // sanity check
  if (count>=max) {
  cout << "blackboard::init - count = " << count << " max = " << max << endl;
	error_exit("blackboard:init","cannot add variable to blackboard");
  }

  if (strlen(name)>BB_NAME_LEN) 
	error_exit("blackboard::init","var name too long");

  strncpy(bb->name,name,strlen(name)+1); 
  bb->size = size;
  bb->cols = cols;
  bb->rows = rows;
  bb->type = type;
}

//----------------------------------------------------------------------------
//	add() 
//	  - add a variable to the blackboard
//	  - several versions of add()s : int, REAL, int *, REAL *, ... etc.
//	  - assume space for table[] was already allocated
//----------------------------------------------------------------------------
// integer scalar 
void blackboard::add(char *name, int *var) {

  BB_Entry *bb = &(table[count]);	// get pointer to the current entry

  init(name,sizeof(int),1,1,BB_INT,bb);	// fill in some basic info
  bb->data.i = var;			// hook up datapointer to actual info
  count++;			

}

//----------------------------------------------------------------------------
// REAL scalar 
void blackboard::add(char *name, REAL *var) {

  BB_Entry *bb = &(table[count]);	// get pointer to the current entry

  init(name,sizeof(REAL),1,1,BB_REAL,bb);// fill in some basic info
  bb->data.r = var;			// hook up datapointer to actual info
  count++;

}

//----------------------------------------------------------------------------
// integer vector [x]
void blackboard::add(char *name, int *var, int cols) {

  BB_Entry *bb = &(table[count]);	// get pointer to the current entry

  init(name,sizeof(int),cols,1,BB_IP,bb);// fill in some basic info
  bb->data.ip = var;			// hook up datapointer to actual info
  count++;

}

//----------------------------------------------------------------------------
// REAL vector [x]
void blackboard::add(char *name, REAL *var, int cols) {

  BB_Entry *bb = &(table[count]);	// get pointer to the current entry

  init(name,sizeof(REAL),cols,1,BB_RP,bb);// fill in some basic info
  bb->data.rp = var;			// hook up datapointer to actual info
  count++;

}

//----------------------------------------------------------------------------
// integer matrix [x][y]
void blackboard::add(char *name, int **var, int cols, int rows) {

  BB_Entry *bb = &(table[count]);	// get pointer to the current entry

  init(name,sizeof(int),cols,rows,BB_IPP,bb);// fill in some basic info
  bb->data.ipp = var;			// hook up datapointer to actual info
  count++;

}

//----------------------------------------------------------------------------
// REAL matrix [x][y]
void blackboard::add(char *name, REAL **var, int cols, int rows) {

  BB_Entry *bb = &(table[count]);	// get pointer to the current entry

  init(name,sizeof(REAL),cols,rows,BB_RPP,bb);// fill in some basic info
  bb->data.rpp = var;			// hook up datapointer to actual info
  count++;

}

//----------------------------------------------------------------------------
//	find() 
//	  - find and return index of named variable
//	    return -1 if not found
//----------------------------------------------------------------------------
int blackboard::find(char *name) {

  // if (!hash_flag) {
  //	return find_brute(name);
  // }

  ENTRY e, *ep;
  char edata[20];
  int	idx;

  e.key = name;

  ep = hsearch(e,FIND);
 
  if (ep!=NULL) {
	// DEBUG : if (find_brute(name)!=atoi(ep->data)) {
  	//		cerr << "blackboard::find - find result differs\n";
	// }
	// cerr << "ep->data = " << ep->data << " in char = " 
	// << (char *)(ep->data) << endl;
	return atoi((char *)ep->data); 
  } 
  return -1;
  
  // brute-force search : I hate it! -- this is extremely slow, and it can
  // become a huge overhead
  /*
  for (int i=0; i<count; ++i) {
	if (strcmp(name,table[i].name) == 0) {
		return (i);
	}	
  }
  return -1;
  */

}

//----------------------------------------------------------------------------
//	find_brute() 
//	  - find and return index of named variable
//	    return -1 if not found
//----------------------------------------------------------------------------
int blackboard::find_brute(char *name) {

  // brute-force search : I hate it! -- this is extremely slow, and it can
  // become a huge overhead
  for (int i=0; i<count; ++i) {
	if (strcmp(name,table[i].name) == 0) {
		return (i);
	}	
  }
  return -1;

}
//----------------------------------------------------------------------------
int blackboard::force_find(char *name) {

  int ret_val = find_brute(name);

  if (ret_val<0) {
	cout << "blackboard::force_find() : " << name << " not found" << endl;
  }
  return ret_val;

}

//----------------------------------------------------------------------------
//	save() 
//	  - save image of blackboard to file : delimit each field with '::'
//	  - each data type must be accompanied with
//		type, size, rows, and cols (all in integer)
//----------------------------------------------------------------------------
int blackboard::save(char *filename) {

# if (BYTEORDER==1234)	
  char *little_endian 	= "BYTEORDER==1234";
# else
  char *big_endian 	= "BYTEORDER==4321";
# endif

  char buf[BB_VER_LEN+BB_COMMENT_LEN+50];

  // open file
  cout << "blackboard : filename = "<< filename <<"\n";
  if (!(fp=fopen(filename,"wb"))) 
	error_exit("blackboard::save","no such blackboard file exists.");

  // write in which byteorder the blackboard is saved
  //	and, the comment
# if (BYTEORDER==1234)	
  strncpy(buf,little_endian,strlen(little_endian)+1); strncat(buf,"::",3);
  strncat(buf,comment,strlen(comment)+1); strncat(buf,"::",3);
# else 			
  strncpy(buf,big_endian,strlen(big_endian)+1); strncat(buf,"::",3);
  strncat(buf,comment,strlen(comment)+1); strncat(buf,"::",3);
# endif
  if (fwrite((void *)buf,sizeof(char),BB_VER_LEN+BB_COMMENT_LEN+50,fp)
	!= BB_VER_LEN+BB_COMMENT_LEN+50) 
	error_exit("blackboard::save","error saving blackboard header");

  // write count
  if(fwrite((void *)&count,sizeof(int),1,fp) != 1) 
	error_exit("blackboard::save","error writing 'count'.");
  
  // use standard C I/O  fwrite
  file_io(BB_WRITE);

  fclose(fp);
  
  return 1;
}

//----------------------------------------------------------------------------
//	load() 
//	  - load image of blackboard from file
//	 	must be tolerant to change of order of the variables
//----------------------------------------------------------------------------
int blackboard::load(char *filename) {
  // use standard C I/O  fread (rev_fread if it is on a little endian arch)

  char buf[BB_VER_LEN+BB_COMMENT_LEN+50];

  // open file
  if (!(fp=fopen(filename,"rb"))) 
	error_exit("blackboard::load","no such blackboard file exists.");

  // load header info
  if (fread((void *)buf,sizeof(char),BB_VER_LEN+BB_COMMENT_LEN+50,fp)
	!= BB_VER_LEN+BB_COMMENT_LEN+50) {
	cout << "number of items read = " << "\n";
	error_exit("blackboard::load","error loading blackboard header");
  }
  buf[17] = '\0';
  printf("endianess: [%s]\n", buf);

  // load count
  int prev_count = count;
  if(fread((void *)&count,sizeof(int),1,fp) != 1) 
	error_exit("blackboard::save","error reading 'count'.");

  // use standard C I/O  fread
  file_io(BB_READ);

  fclose(fp);

  // restore count, in case it was different (in case a derived class
  // load a super classes' image dump file).
  if (prev_count != count)
	count = prev_count;

  return 1;

}

//----------------------------------------------------------------------------
//	file_io() 
//	  - perform actual I/O of the variables (write header, then ...)
//----------------------------------------------------------------------------
void blackboard::file_io(BB_OP io) {

  //int 	nobj, idx, k;
  int	new_idx;

  // save header and data of each blackboard entry
  for (int i=0; i<count; ++i) {

	if ((new_idx = header_io(io,i))<0)
		error_exit("blackboard::file_io","undefined variable.");
	if (var_io(io,new_idx)<0) 
		error_exit("blackboard::file_io","unknown data type.");
	if (i%1000==0) {
		cerr << "blackboard::file_io " << i << "/" << count 
			<< " new_idx = " << new_idx << endl;	
	}

  } 

}

//----------------------------------------------------------------------------
//	header_io() 
//	  - read/write header info. This wouldn't have to be overrided 
//	    by a derived class.
//----------------------------------------------------------------------------
int blackboard::header_io(BB_OP io, int i) { 

  int nobj;
  int idx=0;
  char 	name[BB_NAME_LEN];

  // First, the header for the blackboard entry
  // 	[char name[] | int size | int cols | int rows | int type] 
  //	then save the data 
  // read/write name of variable and find index
  if (io==BB_READ) {

	nobj = (int)fread((void *)name,sizeof(char),
		BB_NAME_LEN,fp);
	if ((idx=this->find(name)) < 0){
		cout << "blackboard::file_io():" << name << "non-existent\n";
		cerr << "    find_brute result = " << find_brute(name) << endl;
		return -1;
	}
	strncpy(table[idx].name,name,strlen(name)+1);

	// read the size/type info
	nobj = (int)fread((void *)&(table[idx].size),sizeof(int),1,fp);
	nobj = (int)fread((void *)&(table[idx].cols),sizeof(int),1,fp);
	nobj = (int)fread((void *)&(table[idx].rows),sizeof(int),1,fp);
	nobj = (int)fread((void *)&(table[idx].type),sizeof(int),1,fp);

	/* debug
	cout << "*** reading idx " << idx << " name = " 
	<< name << " size = " << table[idx].size 
	<< " cols = " << table[idx].cols 
	<< " rows = " << table[idx].rows 
	<< " type = " << (int)(table[idx].type)
	<< endl;
	*/

  } else if (io==BB_WRITE) {

	nobj = (int)fwrite((void *)table[i].name,sizeof(char),
                BB_NAME_LEN,fp);
	idx = i;
	// write the size/type info
	nobj = (int)fwrite((void *)&(table[idx].size),sizeof(int),1,fp);
	nobj = (int)fwrite((void *)&(table[idx].cols),sizeof(int),1,fp);
	nobj = (int)fwrite((void *)&(table[idx].rows),sizeof(int),1,fp);
	nobj = (int)fwrite((void *)&(table[idx].type),sizeof(int),1,fp);

  } else {
	cerr << "**** critical error: unknown file_io mode "
		<< (int)(io) << endl;
  }

  // debug : print what's read and written

  return idx;
}

//----------------------------------------------------------------------------
//	var_io() 
//	  - single variable i/o  returns 0 if certain type is not supported
//	    var_io() can be reused in a subclass of blackboard to support
//	    extended BB_Types.
//	  - suggested usage in a derived class
//		int derived::var_io(int idx) {
//			if (!blackboard::var_io(idx)) {
//				// code for new type
//			        //  (assume header was all read/write)
//			}
//			return supported;
//		}	
//----------------------------------------------------------------------------
int blackboard::var_io(BB_OP io,int idx){ 

  int k, nobj, supported=0;

  // read/write the actual data
  switch (table[idx].type) {

	case BB_REAL :
	  if (io == BB_READ) {
	    nobj = (int)fread((void *)(table[idx].data.r),
		table[idx].size,1,fp); supported=1;
	  } else if (io == BB_WRITE) {
	    nobj = (int)fwrite((void *)(table[idx].data.r),
		table[idx].size,1,fp); supported=1;
  	  }
	  break;

	case BB_RP :
	  if (io == BB_READ) {
	    nobj = (int)fread((void *)(table[idx].data.rp),
		table[idx].size,table[idx].cols,fp); supported=1;
	  } else if (io == BB_WRITE) {
	    nobj = (int)fwrite((void *)(table[idx].data.rp),
		table[idx].size,table[idx].cols,fp); supported=1;
  	  }
	  break;

	case BB_RPP :
	  if (io == BB_READ) {
 	    for (k=0; k<table[idx].cols; ++k)
	      nobj = (int)fread((void *)(table[idx].data.rpp[k]),
		  table[idx].size,table[idx].rows,fp); supported=1;
	  } else if (io == BB_WRITE) {
 	    for (k=0; k<table[idx].cols; ++k)
	      nobj = (int)fwrite((void *)(table[idx].data.rpp[k]),
		  table[idx].size,table[idx].rows,fp); supported=1;
  	  }
	  break;

	case BB_INT : 
	  if (io == BB_READ) {
	    nobj = (int)fread((void *)(table[idx].data.i),
		table[idx].size,1,fp); supported=1;
	  } else if (io == BB_WRITE) {
	    nobj = (int)fwrite((void *)(table[idx].data.i),
		table[idx].size,1,fp); supported=1;
  	  }
	  break;

	case BB_IP :
	  if (io == BB_READ) {
	    nobj = (int)fread((void *)(table[idx].data.ip),
		table[idx].size,table[idx].cols,fp); supported=1;
	  } else if (io == BB_WRITE) {
	    nobj = (int)fwrite((void *)(table[idx].data.ip),
		table[idx].size,table[idx].cols,fp); supported=1;
  	  }
	  break;

	case BB_IPP :
	  if (io == BB_READ) {
 	    for (k=0; k<table[idx].cols; ++k)
	      nobj = (int)fread((void *)(table[idx].data.ipp[k]),
		table[idx].size,table[idx].rows,fp); supported=1;
	  } else if (io == BB_WRITE) {
 	    for (k=0; k<table[idx].cols; ++k)
	      nobj = (int)fwrite((void *)(table[idx].data.ipp[k]),
		table[idx].size,table[idx].rows,fp); supported=1;
  	  }
	  break;

	default : supported = -1; break;

  } // end of switch

  // cerr << " nobj = " << nobj << endl;
  
  return (supported);

}


//----------------------------------------------------------------------------
//	list() 
//	  - list all names in blackboard
//----------------------------------------------------------------------------
void blackboard::list() {

  cout << "blackboard: list of variables\n";
  for (int idx=0; idx<count; ++idx) {
        printf("blackboard[%d] %-15s size=%2d, cols=%4d, rows=%4d, type=",
        	idx,
		table[idx].name,
		table[idx].size,
		table[idx].cols,
		table[idx].rows);
  	printf("%s\n",BB_TYPE_NAMES[table[idx].type]);
  }
  cout << "blackboard: listing complete.\n";

}

//----------------------------------------------------------------------------
//	list_scalar() 
//	  - list all scalar variable names in blackboard
//----------------------------------------------------------------------------
void blackboard::list_scalar() {

  cerr << "blackboard: list of variables\n";
  for (int idx=0; idx<count; ++idx) {
	if (table[idx].type == BB_REAL || table[idx].type == BB_INT) {
           fprintf(stderr,"blackboard[%d] %-15s type=",
        	idx, table[idx].name);
   	   fprintf(stderr,"%s",BB_TYPE_NAMES[table[idx].type]);
	   if (table[idx].type == BB_REAL) { 
	   	fprintf(stderr,", value = %f\n", *(table[idx].data.r));
	   }
	   else {
	   	fprintf(stderr,", value = %d\n", *(table[idx].data.i));
	   }
	}
  }
  cerr << "blackboard: listing complete.\n";

}


//----------------------------------------------------------------------------
//	update() 
//	  - replace data pointer with new one
//----------------------------------------------------------------------------
// REAL matrix [x][y]
void blackboard::update(char *name, REAL **var, int cols, int rows) {

  int idx = find(name);
  if (idx<0) {
	cout << "blackboard::update() : " << name << " is not registered.\n" ;
	return;
  }
  BB_Entry *bb = &(table[idx]);

  init(name,sizeof(REAL),cols,rows,BB_RPP,bb);// fill in some basic info
  bb->data.rpp = var;			// hook up datapointer to actual info

}

//----------------------------------------------------------------------------
//      set_parameter() 
//        - set value of parameter
//----------------------------------------------------------------------------
int blackboard::set_parameter(char *command) {

 // Allowed format 
 //            set <parameter> = <value>
 //            set <parameter>[<column>] = <value>
 //            set <parameter>[<col>][<row>] = <value>

 // 0. since the token "set" has already been consumed, start with the
 //     parameter . User global parser object sim_parser.

 parser *param_parser = new parser(command);

 if (!set_parameter(param_parser)) {
	delete param_parser;
 	return 0;
 }

 delete param_parser;

 return 1;

}
//----------------------------------------------------------------------------
// when parser object is passed as the argument
int blackboard::set_parameter(parser *sim_parser) {

 char param_name[BB_NAME_LEN]="";
 int  col, row;
 int  param_idx;

 if (!sim_parser->match("set")) return 0;

 // 1. match_param handles all three cases and returns
 //             return value            type
 //             col = -1, row = -1 : parameter is a scalar
 //             row = -1, row >= 0 : parameter is a vector
 //             row >= 0, row >= 0 : parameter is a matrix

 // 1.1 match param name and indices
 if (!sim_parser->match_param(param_name, &col, &row)) return 0;

 // 1.2 match "="
 if (!sim_parser->force_match("=")) return 0;

 // 1.3 get parameter index from table
 if ((param_idx=force_find(param_name))<0) return 0;

 // 1.4 set value
 int int_val; REAL real_val;
 switch (table[param_idx].type) {

        case BB_INT: sim_parser->scan_int(&int_val);
                *(table[param_idx].data.i) =  int_val;
                break;
        case BB_IP: sim_parser->scan_int(&int_val);
		if (col>=table[param_idx].cols) 
			warning("balckboard::set_param",
				"index out of range : given %d, max %d",
				col, table[param_idx].cols);
		else
                	table[param_idx].data.ip[col] = int_val;
                break;
        case BB_IPP: sim_parser->scan_int(&int_val);
		if (col>=table[param_idx].cols || 
		    row>=table[param_idx].rows ) 
			warning("balckboard::set_param",
				"index out of range : given %d:%d, max %d:%d",
				col, row,
				table[param_idx].cols,
				table[param_idx].rows);
		else
                	table[param_idx].data.ipp[col][row] = int_val;
                break;
        case BB_REAL: sim_parser->scan_real(&real_val);
                *(table[param_idx].data.r) =  real_val;
                break;
        case BB_RP: sim_parser->scan_real(&real_val);
		if (col>=table[param_idx].cols) 
			warning("balckboard::set_param",
				"column out of range : %d >= %d",
				col, table[param_idx].cols);
		else
                	table[param_idx].data.rp[col] = real_val;
                break;
        case BB_RPP: sim_parser->scan_real(&real_val);
		if (col>=table[param_idx].cols || 
		    row>=table[param_idx].rows ) 
			warning("balckboard::set_param",
				"index out of range : given %d:%d, max %d:%d",
				col, row,
				table[param_idx].cols,
				table[param_idx].rows);
		else 
                	table[param_idx].data.rpp[col][row]=real_val;
                break;
	default : return 0;
   }

   return 1;
}

//----------------------------------------------------------------------------
//	add_hash() 
//	  - add entry to hash table
//		name : name of variable
//		count : index in table[]
//----------------------------------------------------------------------------
void blackboard::add_hash(char *name, int count) {
  ENTRY e;

  e.key = name;

  sprintf((char *)hashtab[count].key,"%s",name);
  sprintf((char *)hashtab[count].data,"%d",count);
  if (!hsearch(hashtab[count],ENTER)) {
	cerr <<"blackboard::add_hash-failed : memory full" << endl;
  }
  if (!hsearch(e,FIND)) {
	cerr <<"blackboard::add_hash-failed adding " << hashtab[count].key << endl;
	cerr <<"blackboard::add_hash-failed adding " << hashtab[count].data<< endl;
  }
}

//----------------------------------------------------------------------------
//	build_hash() 
//	  - build hash table
//----------------------------------------------------------------------------
void blackboard::build_hash() {

  if (!hcreate(max)) {
	error_exit("blackboard::load","hcreate() failed\n");
  }

  hash_flag = 1;

  cerr << "blackboard::build_hash() " << comment << " - begin\n";
  for (int i=0; i<count; ++i) {

	add_hash(table[i].name, i);
  }
  cerr << "blackboard::build_hash() - end\n";

}
//----------------------------------------------------------------------------
void blackboard::destroy_hash() {

  hdestroy();

  hash_flag = 0;

}



