//----------------------------------------------------------------------------
// 
//	file: mattest.cc
// 
//	function: test if matrex different indexing affects performance 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Sun Sep  6 01:31:06 CDT 1998
// 
//	$Revision: 1.1 $ $Date: 1998/10/16 10:35:21 $
//	$Log: mattest.cc,v $
//	Revision 1.1  1998/10/16 10:35:21  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------

#include <iostream.h>
#include <nnview/simulog.h>

int main(int argc, char *argv[]) {

  simulog *matlog = new simulog("mattest.cc : test difference in indexing",
				"mattest.log");

  
  if (argc!=3) {
	cout << "usage: mattest N m\n    N : size of matrix NxN\n";
 	cout << "    m : number of iterations\n";
	exit(1);
  }

  int N = atoi(argv[1]);
  int m = atoi(argv[2]);
  cout << "matrix = " << N << "x" << N << endl;

  //
  // Let's suppose we have an matrix mat[N/3..2N/3][N/3..2N/3]
  //

  //
  // 1. First, calculate the ranges.
  //
  int lowx = N/3; int highx = 2*N/3; int cols = highx-lowx+1;
  int lowy = N/3; int highy = 2*N/3; int rows = highy-lowy+1;

  //
  // 2. Allocate matrix, Mvanilla and Mdirect.
  //	   Mvanilla[x][y] has range [0..cols-1][0..rows-1].
  //	   Mdirect[x][y] has range [lowx..highx][lowy..highy]
  double **Mvanilla = new double* [cols];
  double **Mdirect  = new double* [cols];
  Mdirect  = &Mdirect[-lowx];
  for (int i=0; i<cols; ++i) {
	Mvanilla[i] = new double [rows];
	Mdirect[i+lowx] = new double [rows];
	Mdirect[i+lowx] = &Mdirect[i+lowx][-lowy];
  }

  //
  // 3. Now test if the indexing scheme works.
  //
  matlog->append("Start vanilla matrix crunching");

  for (int k=0; k<m; ++k) {
    for (int j=lowy; j<=highy; ++j) {
   	for (int i=lowx; i<=highx; ++i) {
		Mvanilla[i-lowx][j-lowy] = i+lowx + (j+lowy)*10.0;
		// Mdirect[i+lowx][j+lowy] = i+lowx + (j+lowy)*10.0;
		//cout << Mvanilla[i][j] << ":" << int(&Mvanilla[i][j]) << " ";

	}
	//cout << endl;
    }
  }
  matlog->append("End vanilla matrix crunching");
  //cout << "==================================\n";
  matlog->append("Start direct access matrix crunching");
  for (int k=0; k<m; ++k) {
    for (int j=lowy; j<=highy; ++j) {
   	for (int i=lowx; i<=highy; ++i) {
		Mdirect[i][j] = i+lowx + (j+lowy)*10.0;
		//cout << Mdirect[i][j] << ":" << int(&Mdirect[i][j]) << " ";
	}
	//cout << endl;
    }
  }
  matlog->append("End direct access matrix crunching");

  // clean up and exit
  delete matlog;

  return 0;

}
  
