## usage: [Smax,sVmax,Eimg] = convspaceplot(Img, kernelsize, theta)
##
##	Img: p x q matrix (grayscale image)
##
##		Img: 2D matrix with grayscale image
##		kernelsize: width (==height) of convolution kernel
##		theta: gabor filter orientation 
##
##
## First, convolve with DoG filter, and then convolve with gabor
## filters of frequencies 1, 2, 3, ... , 10. 
##
##	Usage example:
##		[Smax,sVmax,Eimg] = convspaceplot(imagemat,9,0);
##
##		both DoG and gabor filters are  9 x 9, and
##		Gabor filter orientation is 0.
##		final grid sampling size is 3 x 3.
##
## $Id: convedgemat.m,v 1.2 2002/03/09 17:31:27 yschoe Exp yschoe $

function [Smax,sVmax,Eimg] = convspaceplot(Img, kernelsize, theta)

	# get image size
	[oldp, oldq] = size(Img);

	# adjust grid size to have the conv kernels
	# centered at the middle: i.e. make the gridsize an odd number.
	if ( rem(kernelsize,2) == 0 ) 
		kernelsize ++;
	endif

	DoG = dog(kernelsize);

        # convolve with DoG, and only use the real part
	Eimg = real(conv2(Img,DoG,"same"));	
  
       	sVmax=zeros(oldp,oldq);
        Smax=zeros(oldp,oldq);
       
	g = gabor(kernelsize,kernelsize,"sin",theta,1);

        sVmax=real(conv2(Eimg,g,"same"));
	Smax=Smax+1;
        
        #find sVmax and Smax iteratively
	for k=2:6
             g = gabor(kernelsize,kernelsize,"sin",theta,k);
             sVmax=max(sVmax,real(conv2(Eimg,g,"same")));
             Smax=max(Smax,k*(sign((real(conv2(Eimg,g,"same")))-sVmax)+1));
        endfor


endfunction
