## Copyright (C) 2000 uschoe
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2, or (at your option)
## any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, write to the Free
## Software Foundation, 59 Temple Place - Suite 330, Boston, MA
## 02111-1307, USA.

## usage: kurtosis2d (X)
##
## Returns a kurtoris value for a matrix X
##

## Author: Yoonsuck Choe <yschoe@cs.utexas.edu>

function k = kurtosis2d (X)

  # calculate center of gravity 
  cogx = 0;
  cogy = 0;
  n = columns(X);

  for x=1:n ;
  	for y=1:n ;
		cogx = cogx + x*X(x,y);	
		cogy = cogy + y*X(x,y);	
  	endfor
  endfor

  s = sum(sum(X));

  cogx = cogx / s;
  cogy = cogy / s;

  k = 0;
  for i=1:n ;
	for j=1:n ;
		k = k + ((i-cogx)**2+(j-cogy)**2)**2*X(i,j);
	endfor
  endfor

endfunction
