# function [c] = msonlinefit (dim, x, d, eta, iter, [c]) 
#	fit a cubic polinomial to input->output mapping of x_i --> d_i
#	learning rate : eta
#	c : initial coefficient vector
#	e : final error
#	Example: 
#	[c] = msonlinefit(1, (1:10),(1:10)+2*rand(1,10), 0.001,  100000, [ 10 -1]')
function [c] = msonlinefit (deg, x, d, eta, iter, c) 

  # init coefficients
  if (nargin<6) 
  	c = rand(deg+1,1)*max(x);
  endif

  # transpose row vectors.
  if (columns(x)>1) 
	x = x';
  endif

  if (columns(d)>1) 
	d = d';
  endif

  if (columns(c)>1) 
	c = c';
  endif

  # common constants
  numinp = length(x);
  disp = 100;
  prev_error = 1000;

  # pre-generate input matrix  (each row is one input)
  for n=1:numinp
		oneinp = [1];
		# generate powers
		for i=1:deg
			oneinp = [oneinp, x(n)**i];
		endfor
		if (n==1)
			inp = oneinp;
		else
			inp = [inp ; oneinp ];
		endif
  endfor 

  # repeat epoch

  numcoeff = deg+1;

  for epc = 1:iter

	# get deltac
	deltac = zeros(deg+1,1);
	for n=1:numinp
  		out(n) = inp(n,:)*c;
		for i=1:numcoeff
			deltac(i) = deltac(i) + (out(n)-d(n)) *(x(n)**(i-1));
		endfor
		c = c - 2*eta*deltac/numinp;
	endfor


	# plot
	if (rem(epc,disp)==0) 
		plot(x,d, "x" ,x,out, "o-");
	endif

  endfor

endfunction
