#!/usr/bin/wish
#----------------------------------------------------------------------------
# usage: itest.tcl
#
#	pronounciation: eye test
#
#		Draw a white disc with random (optional) radius
#		at the center of a black screen.
#		Calculate average area of boundary drawn by human subject
#		and the true area in each trial.	
#		Repeat until the [X] exit button on the upper right
#		corner is pressed.
#	
#	required software
#		- you need Tcl/Tk interpreter from www.scriptics.com
#		- once you install tcl/tk, this file will be executable
#		  by double clicking on the icon.
#
# Author: Yoonsuck Choe <choe@cs.tamu.edu>
#
# Copyright: all rights reserved by Yoonsuck Choe (2001)
# Licensing terms: 
#	Permission of the author is required for commercial use.
#	For non-commercial use, notification via email to the author
#	is required.	
# Warranty:	No warranty, use at own risk.
# $Id: itest.tcl,v 1.1 2002/09/05 19:37:31 yschoe Exp yschoe $
#----------------------------------------------------------------------------
# Configuration : modify to your taste 
#----------------------------------------------------------------------------
global maxDiscSize randFlag

set randFlag    1	; # randomize size? 1=yes, 0=no
set maxDiscSize 100	; # max radius of disk (pixels): default when randFlag=0
set minDiscSize 30	; # min radius of disk (pixels)
set borderColor red	; # default pen color

#----------------------------------------------------------------------------
# End of configurable data
#----------------------------------------------------------------------------

set myfg 	\#aa0000		; # body and titlebar text color
set mybg 	\#000000 		; # body background
set mytitlefg 	\#aa0000		; # titlebar button text color 
set mytitlebg 	\#000000 		; # titlebar background

global offsetx offsety

set width	[winfo screenwidth .]
set height 	[winfo screenheight .]
set geometry 	"${width}x${height}-0-0"		; # geometry

global centx centy
set centx	[expr $width/2]
set centy	[expr $height/2]

global pressedFlag
set pressedFlag 0

global pointList areaList trueAreaList

set areaList [list]
set trueAreaList [list]

#----------------------------------------------------------------------------
#	default colors for windows and widgets
#----------------------------------------------------------------------------
option add *activeForeground white
option add *activeBackground black
option add *foreground white
option add *background black
option add *borderwidth 0

#----------------------------------------------------------------------------
# procedure: drag_move_setup : setup binding so that window can be dragged
#	and moved
#----------------------------------------------------------------------------
proc drag_move_setup { toptitle } {

  bind $toptitle <ButtonPress-1> {
	  set offsetx [expr [winfo pointerx .calendar ]-[winfo x .calendar]]
	  set offsety [expr [winfo pointery .calendar ]-[winfo y .calendar]]
	  bind $toptitle <Motion> {
		set lastx [expr [winfo pointerx .calendar ]-$offsetx]
		set lasty [expr [winfo pointery .calendar ]-$offsety]
		wm geometry .calendar "+${lastx}+${lasty}"
	  }
	}
  bind $toptitle <ButtonRelease-1> {
	set lastx [expr [winfo pointerx .calendar ]-$offsetx]
	set lasty [expr [winfo pointery .calendar ]-$offsety]
	wm geometry .calendar "+${lastx}+${lasty}"
	# Clear binding to the <Motion> event
	bind $toptitle  <Motion> {}
	}

  bind $toptitle <Enter> {
	$toptitle config -cursor fleur
	}

}

#----------------------------------------------------------------------------
#	drawCircle
#----------------------------------------------------------------------------
proc draw_circle { canvwin } {
	global centx centy trueAreaList maxDiscSize minDiscSize randFlag
	if {$randFlag} {
		set rad [expr int(rand()*$maxDiscSize)]
		if {$rad < $minDiscSize} {
			set rad $minDiscSize
		}
	} else {
		set rad $maxDiscSize
	}
	$canvwin create oval \
		[expr $centx-$rad] [expr $centy-$rad]  \
		[expr $centx+$rad] [expr $centy+$rad]  \
 		-outline white -fill white -tags [list circs]
	lappend trueAreaList [list [expr 3.141592*$rad*$rad]]
}

#----------------------------------------------------------------------------
#	drawLine
#----------------------------------------------------------------------------
proc draw_line { canvwin sx sy ex ey} {
	global borderColor
	$canvwin create line \
		$sx $sy $ex $ey\
 		-fill $borderColor -tags [list lines]
}
#----------------------------------------------------------------------------
#	calaArea : use polygon area calculation algorithm
#----------------------------------------------------------------------------
proc calc_area { pointlist } {

   # ref: http://astronomy.swin.edu.au/pbourke/geometry/polyarea/

   set N [llength $pointlist]

   set area 0

   for {set i 0} {$i<$N} {incr i} {
	set j [expr ($i+1)%$N]
	set itemi [lindex $pointlist $i]
	set itemj [lindex $pointlist $j]
	set ix [lindex $itemi 0]
	set iy [lindex $itemi 1]
	set jx [lindex $itemj 0]
	set jy [lindex $itemj 1]
        set area [expr $area + $ix * $jy]
        set area [expr $area - $iy * $jx]
   }

   set area [expr $area/2.0];
   if {$area<0} {
	set area [expr -$area]
   }
   return $area
}

#----------------------------------------------------------------------------
#	DialogBox
#----------------------------------------------------------------------------
proc DialogBox { id message } {
  
  toplevel .dialogTemp$id 
  wm title .dialogTemp$id "Confirm"
  wm geometry .dialogTemp$id +200+200
  frame .dialogTemp$id.f ; pack .dialogTemp$id.f
  # label .dialogTemp$id.f.icon -bitmap @warnin
  # pack .dialogTemp$id.f.icon -padx 3m -pady 3m -side left
  label .dialogTemp$id.f.msg -text  "$message" -justify left
  pack .dialogTemp$id.f.msg -padx 3m -pady 3m -side left
  button .dialogTemp$id.ok -text "OK" -command "destroy .dialogTemp$id; return"
  pack .dialogTemp$id.ok -padx 5m -pady 3m
  tkwait window .dialogTemp$id
  exit
  
} 


#----------------------------------------------------------------------------
wm withdraw . 

option add *activeBackground $mybg

#----------------------------------------
# Create new window
#----------------------------------------
toplevel .calendar -bg $mybg -relief flat 
wm geometry .calendar $geometry
wm overrideredirect .calendar 1

#----------------------------------------
# Top frame
#----------------------------------------
# frame .calendar.top -relief flat
# pack .calendar.top -fill both 

set top .calendar

#----------------------------------------
# Title frame
#----------------------------------------
frame $top.titleframe -bg $mytitlebg  -relief flat -borderwidth 0
pack $top.titleframe -side top -fill both
drag_move_setup $top.titleframe

label $top.titleframe.title -text "iTest" -bg $mytitlebg -fg $myfg \
	-height 1 -relief flat -borderwidth 0
# pack this later
# the title is packed in the end
pack $top.titleframe.title -side left -fill both -padx 0 -expand 1 -ipadx 0 -ipady 0

set toptitle $top.titleframe.title

# Since overridedirect disables any direct influence of the window manager
# on this application, provide a primitive way of moving around the 
# application window.

drag_move_setup $toptitle

#----------------------------------------
# Body
#----------------------------------------
canvas $top.canv -relief flat -borderwidth 0 -background black\
        -width $width -height $height  \
	-highlightbackground black
	
pack $top.canv -side top -fill both -ipadx 0 -ipady 0

# Change cursor shape when it enters the main window and
# draw a white disk of random radius
bind $top.canv <Enter> {
	$top.canv config -cursor crosshair;
	draw_circle $top.canv
}

# When button is pressed, start drawing outline
bind $top.canv <ButtonPress-1> {
	global pressedFlag xpos ypos newxpos newypos pointList
	set xpos %x 
	set ypos %y
	set pressedFlag 1
	set pointList [list [list $xpos $ypos]]
}

bind $top.canv <Motion> {
	global pressedFlag xpos ypos newxpos newypos pointList
	if {$pressedFlag} {
		set newxpos %x;	
		set newypos %y;	
		draw_line $top.canv $xpos $ypos $newxpos $newypos
		set xpos $newxpos;	
		set ypos $newypos;	
		lappend pointList [list $xpos $ypos]
	}
}

# When button is released, 
bind $top.canv <ButtonRelease-1> {

	global pressedFlag xpos ypos newxpos newypos pointList

	$top.canv delete allItems withtag circs
	$top.canv delete allItems withtag lines
	draw_circle $top.canv 
	set pressedFlag 0
	
	set area [calc_area $pointList]
	puts "area = $area";
	lappend areaList [list $area]
}

#----------------------------------------
# Buttons in Title frame 
#----------------------------------------
frame $top.buttons -relief flat -borderwidth 0 -highlightthickness 0 -highlightcolor black
pack $top.buttons -side top -fill both -ipadx 0 -ipady 0

set topt $top.titleframe

#--------------------
# Quit button : report result before quitting
#--------------------
button $topt.quit -text "X" -bg $mytitlebg -fg \#aa0000 -font 5x7 -relief flat \
 	-borderwidth 0 -padx 5  -command {
		global areaList trueAreaList
		set sum 0;
		set count 0;
		foreach area $areaList {
			incr count
			set sum [expr $sum + $area ]
		}
		set trueSum 0;
		for {set i 0} {$i<$count} {incr i} { 
			set trueArea [lindex $trueAreaList $i]
			set trueSum [expr $trueSum + $trueArea]
		}
		set str "Human: total = $sum, samples = $count, avg area = [expr $sum/$count]\n\nTrue: total = $trueSum, samples = $count, avg area = [expr $trueSum/$count]\n\nAvg err = [expr $trueSum/$count-$sum/$count]\n\nErr Percentage = [expr abs(($trueSum-$sum)/$trueSum)]%"
		DialogBox exit $str
  	}

#--------------------
# pack buttons in order
#--------------------
pack $topt.quit -padx 0 -pady 0 -side right -fill both -ipadx 0 -ipady 0

bind $topt.quit <Enter> { $topt.quit config -cursor hand2 }
